# frozen_string_literal: true

RSpec.describe PlanetExpress::ShippingOptionRepository do
  subject { PlanetExpress::ShippingOptionRepository.new }

  describe '#where' do
    context 'no data' do
      subject { PlanetExpress::ShippingOptionRepository.new(data: []) }

      it 'returns an empty array' do
        expect(subject.where()).to eq([])
      end
    end

    context 'no arguments' do
      before(:each) do
        expect(subject.all).to_not be_empty
      end

      it 'returns all items' do
        expect(subject.where()).to eq(subject.all)
      end
    end

    context 'with existing attribute' do
      let(:expected_item) do
        build(:shipping_option, package_size: 'L', price_in_cents: 410)
      end

      let(:data) do
        build_list(:shipping_option, 4, price_in_cents: 500) + [expected_item]
      end

      subject do
        PlanetExpress::ShippingOptionRepository.new(data: data)
      end

      it 'returns a list with the expected item' do
        expect(subject.where(package_size: 'L', price_in_cents: 410)).to eq([expected_item])
      end
    end

    context 'with unknown attribute' do
      it 'throws an exception' do
        expect do
          subject.where(unknown_attribute: nil)
        end.to raise_error(NoMethodError)
      end
    end
  end

  describe '#lowest_price_for_package_size' do
    it 'finds the lowest price for a package size' do
      expect(subject.lowest_price_for_package_size('L')).to eq(400)
    end
  end

  describe '#lowest_price_for_package_size' do
    context 'existing option' do
      it 'finds the correct shipping option' do
        result = subject.find_by_provider_and_package_size('LP', 'S')

        expect(result.provider).to eq('LP')
        expect(result.package_size).to eq('S')
        expect(result.price_in_cents).to eq(150)
      end
    end

    context 'unknown option' do
      it 'returns nil' do
        result = subject.find_by_provider_and_package_size('UNKNOWN', 'S')

        expect(result).to eq(nil)
      end
    end
  end
end
